function getReason(reasonEventEnum, reasonId)
{
	for(var reasonName in ic.Reason[reasonEventEnum])
	{
		if (ic.Reason[reasonEventEnum][reasonName] === reasonId)
			return 'EVENTINFO_REASON_'+reasonEventEnum+'_'+reasonName;
	}
	return 'EVENTINFO_REASON_'+reasonId;
}

function logsPage_licenseErrorLocal( reason )
{
	return _res('LICENSE_ERROR_' + reason);
}

function logsPage_licenseErrorPair( reason )
{
	return [ _res('EVENTINFO_REASON'), logsPage_licenseErrorLocal(reason) ];
}

function logsPage_formatTime(sec)
{
	return ic.timeToString(sec);
}

function logsPage_FormatObject(data)
{
	var style = 'style="border:none;padding:3px"';

	function formatRow(data)
	{
		if(data.length == 1)
			return('<td '+style+' class="nobr" colspan="2">' + data[0] + '</td>');

		if(data.length == 2)
			return(
				'<td ' + style + ' class="nobr">' + data[0]
				+ ':</td>'
				+ '<td ' + style + ' class="w100">'
				+ data[1]
				+ '</td>' );
	}

	res = '<table class="w100">';
	for(var i = 0; i < data.length; i++)
		res += '<tr>' + formatRow(data[i]) + '</tr>';
	res += '</table>';
	return res;
}


var logsPage_getInfo = {}

logsPage_getInfo.TaskStateChanged = function(data)
{
	var stateName = getObjectKeyByValue(ic.TaskState, data.TaskState);
	return _res('EVENTINFO_TaskStateChanged_' + stateName);
}

logsPage_getInfo.AVBasesAttached = function(data)
{
	data.basesDate = logsPage_formatTime(data.AVBasesDate);
	return new Template(_res('EVENTINFO_AVBasesAttached')).evaluate(data);
}

logsPage_getInfo.TaskSettingsChanged = function(){}

logsPage_getInfo.AVBasesAreOutOfDate = function(data)
{
	return new Template(_res('EVENTINFO_AVBasesAreOutOfDate')).evaluate(data);
}

logsPage_getInfo.AVBasesAreTotallyOutOfDate = logsPage_getInfo.AVBasesAreOutOfDate;

logsPage_getInfo.UpdateSourceSelected = function(data)
{
	return data.Source.UpdateUrl; // FIXME: make more precise
}

logsPage_getInfo.UpdateSourceNotSelected = function(data)
{
	return logsPage_FormatObject([
		[ data.Source.UpdateUrl ],
		[ _res('EVENTINFO_REASON'), data.SourceReason ]
	]);
}

logsPage_getInfo.NothingToUpdate = function(){}

logsPage_getInfo.ModuleDownloaded = function(data)
{
	return data.ModuleName;
}

logsPage_getInfo.ModuleNotDownloaded = function(data)
{
	return logsPage_FormatObject([
		[ data.ModuleName ],
		[ _res('EVENTINFO_REASON'), data.ModuleReason ]
	]);
}

logsPage_getInfo.AVBasesIntegrityCheckOK = function(){}

logsPage_getInfo.AVBasesIntegrityCheckFailed = function(data)
{
	return logsPage_FormatObject([
		[ _res('EVENTINFO_REASON'), data.IntegrityReason ]
	]);
}

logsPage_getInfo.ModuleRetranslated = function(data)
{
	return data.ModuleName;
}

logsPage_getInfo.ModuleNotRetranslated = function(data)
{
	return logsPage_FormatObject([
		[ data.ModuleName ],
		[ _res('EVENTINFO_REASON'), data.ModuleReason ]
	]);
}

logsPage_getInfo.UpdateError = function(data)
{
	return logsPage_FormatObject([
		[ data.ModuleName ],
		[ _res('EVENTINFO_REASON'), data.ErrorReason ]
	]);
}

logsPage_getInfo.RetranslationError = logsPage_getInfo.UpdateError;

logsPage_getInfo.AVBasesRollbackCompleted = function(){}

logsPage_getInfo.AVBasesRollbackError = function(data)
{
	return logsPage_FormatObject([
		[ _res('EVENTINFO_REASON'), data.RollbackReason ]
	]);
}

logsPage_getInfo.LicenseInstalled = function(data)
{
	return logsPage_FormatObject([
		[ _res('EVENTINFO_LICENSE_KEYSERIAL'), data.KeySerial ]
	]);
}

logsPage_getInfo.LicenseNotInstalled = function(data)
{
	return logsPage_FormatObject([
		[ _res('EVENTINFO_LICENSE_KEYSERIAL') , data.KeySerial ],
		logsPage_licenseErrorPair(data.ErrorReason)
	])
}

logsPage_getInfo.LicenseRevoked = logsPage_getInfo.LicenseInstalled;

logsPage_getInfo.LicenseNotRevoked = logsPage_getInfo.LicenseNotInstalled;

logsPage_getInfo.LicenseExpired = function(){}

logsPage_getInfo.LicenseExpiresSoon = function(data)
{
	return new Template(_res('EVENTINFO_LicenseExpiresSoon'))
		.evaluate(data);
}

logsPage_getInfo.LicenseError = function(data)
{
	var reasonData = [ logsPage_licenseErrorLocal(data.ErrorReason) ];
	if('Unknown' == data.FuncLevel)
		return logsPage_FormatObject([ reasonData ]);
	else
		return logsPage_FormatObject([ reasonData, ['Level', data.FuncLevel] ]);
}

logsPage_getInfo.PackedObjectDetected = function(data)
{
	return logsPage_FormatObject([
		[_res('EVENTINFO_OBJECTNAME'), misc_formatLongName(data.Object.FileName) ],
		[_res('EVENTINFO_ACCESSHOST'), data.Object.AccessHost ],
		[_res('EVENTINFO_PACKER'),     data.PackerName ]
	])
}

logsPage_getInfo.ThreatDetected = function(data)
{
	var res = [
		[ _res('EVENTINFO_OBJECTNAME'), misc_formatLongName(data.Object.FileName) ],
		[ _res('EVENTINFO_THREATNAME'), data.ThreatName ]
	];

	if(data.DangerLevel != 'Unknown')
		res.push([
			_res('EVENTINFO_DANGERLEVEL'),
			_res('EVENTINFO_DANGERLEVEL_' + data.DangerLevel)
		]);

	if(data.DetectCertainty != 'Unknown')
		res.push([
			_res('EVENTINFO_DETECTCERTAINTY'),
			_res('EVENTINFO_DETECTCERTAINTY_' + data.DetectCertainty)
		]);

	if(data.VerdictType != 'Undefined')
		res.push([
			_res('EVENTINFO_VERDICTTYPE'),
			_res('EVENTINFO_VERDICTTYPE_' + data.VerdictType)
		]);

	return logsPage_FormatObject(res);
}

function processObject(data, reasonRes)
{
	return logsPage_FormatObject([
		[_res('EVENTINFO_OBJECTNAME') , misc_formatLongName(data.Object.FileName) ],
		[_res('EVENTINFO_ACCESSHOST') , data.Object.AccessHost ],
		[_res('EVENTINFO_REASON')     , _res(getReason(reasonRes, data.ObjectReason)) ]
	])
}

logsPage_getInfo.ObjectProcessed =        function(data){ return processObject(data, 'ObjectProcessedReason') };
logsPage_getInfo.ObjectActionsCompleted = function(data){ return processObject(data, 'ObjectActionsCompletedReason') };
logsPage_getInfo.ObjectNotProcessed =     function(data){ return processObject(data, 'ObjectNotProcessedReason') };


function processObjectError(data, reasonRes)
{
	if(typeof(data.Object.FileName) !== 'undefined') // AV event
		return processObject(data, 'ObjectNotDisinfectedReason');

	if(typeof(data.Object.FileName) !== 'undefined') // QS event
		return logsPage_FormatObject([
			[_res('EVENTINFO_OBJECTNAME') , misc_formatLongName(data.Object.FileName) ],
			[_res('EVENTINFO_REASON')     , _res(getReason(reasonRes, data.ObjectReason)) ],
			['ID', data.Object.ID ]
		])

	return Object.toJSON(data);
}

logsPage_getInfo.ObjectNotDisinfected =  function(data){ return processObjectError(data, 'ObjectNotDisinfectedReason') };
logsPage_getInfo.ObjectProcessingError = function(data){ return processObjectError(data, 'ObjectProcessingErrorReason') };

logsPage_getInfo.ObjectDisinfected = function(data)
{
	return logsPage_FormatObject([
		[_res('EVENTINFO_OBJECTNAME'), misc_formatLongName(data.Object.FileName) ],
		[_res('EVENTINFO_ACCESSHOST'), data.Object.AccessHost ]
	])
}

logsPage_getInfo.ObjectDeleted = logsPage_getInfo.ObjectDisinfected;

logsPage_getInfo.ObjectBlocked = function(data)
{
	return logsPage_FormatObject([
		['Host: '      , data.Host ],
		[_res('EVENTINFO_OBJECTNAME') , misc_formatLongName(data.Object.FileName) ],
		[_res('EVENTINFO_ACCESSHOST') , data.Object.AccessHost ]
	])
}

logsPage_getInfo.AVBasesApplied =
logsPage_getInfo.ApplicationStarted =
logsPage_getInfo.ApplicationShutdown =
logsPage_getInfo.ApplicationSettingsChanged =
logsPage_getInfo.EventsMoved =
logsPage_getInfo.EventsErased = function(){}

/*******************************
 * Quarantine-related functions
 *******************************/
logsPage_getInfo.ObjectRemovedFromQuarantine  =
logsPage_getInfo.ObjectRemovedFromBackup  =
logsPage_getInfo.ObjectRestoredFromQuarantine  =
logsPage_getInfo.ObjectRestoredFromBackup  =
logsPage_getInfo.ObjectAddToQurantineFailed  =
logsPage_getInfo.ObjectRestored =
logsPage_getInfo.SizeLimitReached =
logsPage_getInfo.SoftSizeLimitExceeded =
logsPage_getInfo.ObjectAddToBackupFailed =
logsPage_getInfo.ObjectCurable   =
logsPage_getInfo.FalseDetect     =
logsPage_getInfo.Corrupted       =
logsPage_getInfo.ObjectAddedToQuarantine =
logsPage_getInfo.ObjectAddedToBackup = function(data)
{
	return misc_formatLongName(data.FileName);
}

logsPage_getInfo.OASTaskError = function(data)
{
	var result = '';
	['Unknown',
	 'IncorrectUser',
	 'IncorrectGroup',
	 'IncorrectPath',
	 'InterceptorNotFound'].each(function(val){
		if(data.Error == ic.OASTaskErrorReason[val])
			result = _res('OASTaskErrorReason_' + val);
	})
	result += ' ' + data.Info;
	return result;
}

logsPage_getInfo.ODSTaskError = function(data)
{
	var result = '';
	['Unknown','IncorrectPath'].each(function(val){
		if(data.Error == ic.ODSTaskErrorReason[val])
			result = _res('ODSTaskErrorReason_' + val);
	})
	result += ' ' + data.Info;
	return result;
}

var logsPage_rowTmpl = new Template(
	'<tr>'+
	'<td>#{name}</td>'+
	'<td>#{severity}</td>'+
	'<td><span title="#{taskNameFull}">#{taskName}</span></td>'+
	'<td>#{info}</td>'+
	'<td class="nobr">#{date}</td>'+
	'</tr>');

function logsPage_fireResultItem(item)
{
	item.date = logsPage_formatTime(item.date);

	if(logsPage_getInfo[item.name])
		item.info = logsPage_getInfo[item.name](item.data);
	else
		item.info = Object.toJSON(item.data)

	var severity  = getObjectKeyByValue(
		ic.EventStorageSeverity,
		item.severity,
		ic.EventStorageSeverity.Unknown);
	item.severity = _res('LOGS_SEVERITY_' + severity);

	if(item.name == 'ThreatDetected')
		item.name = '<span style="color:red;font-weight:bold">'
			+ _res('EVENTNAME_' + item.name)
			+ '</span>';
	else
		item.name = _res('EVENTNAME_' + item.name);

	if(item.data.PersistentTaskId)
		item.taskId = item.data.PersistentTaskId;
	item.taskNameFull = misc_preparetoView(item.taskName);
	item.taskName = misc_preparetoView(item.taskName.length > 40 ? item.taskName.substr(0,40)+'...':item.taskName);

	return logsPage_rowTmpl.evaluate(item);
}

ic.registerConnectionErrorHandler(function(connLostId){
	if(0 == $('#logsPageMarker').length )
		return;

	if(connectLostState.connLostId == connLostId)
		return;

	connectLostState = {
		connLostId      : connLostId,
		refreshButton   : ($('a#refreshButton:not(.disabledLink)').length > 0),
		firstPageButton : ($('#firstPageButton:enabled').length > 0),
		prevPageButton  : ($('#prevPageButton:enabled').length > 0),
		nextPageButton  : ($('#nextPageButton:enabled').length > 0),
		lastPageButton  : ($('#lastPageButton:enabled').length > 0)
	};

	if(connectLostState.refreshButton) searchForm.toggleRefreshButtonState(false);

	if(connectLostState.firstPageButton) $('#firstPageButton').disable();
	if(connectLostState.prevPageButton)  $('#prevPageButton').disable();
	if(connectLostState.nextPageButton)  $('#nextPageButton').disable();
	if(connectLostState.lastPageButton)  $('#lastPageButton').disable();

	$('#resultTable :checkbox').disable();
	$('#searchButton').disable();
	$('#resultTable a').disable();
})

ic.registerConnectionRestoredHandler(function(){
	if(0 == $('#logsPageMarker').length)
		return;

	if(connectLostState.refreshButton) searchForm.toggleRefreshButtonState(true);

	if(connectLostState.firstPageButton) $('#firstPageButton').enable();
	if(connectLostState.prevPageButton)  $('#prevPageButton').enable();
	if(connectLostState.nextPageButton)  $('#nextPageButton').enable();
	if(connectLostState.lastPageButton)  $('#lastPageButton').enable();

	$('#resultTable :checkbox').enable();
	$('#searchButton').enable();
	$('#resultTable a').enable();
})
