function scopePage_ViewDataSynchronizer()
{
	var this_ = this;

	var convertData = [
		// [ scopename, fstype, fsname ]
		[ 'SCOPE_LABEL_LOCAL',       ic.FSType.Local,   ic.FSName.Any   ],

		[ 'SCOPE_LABEL_MOUNTED_SMB', ic.FSType.Mounted, ic.FSName.Samba ],
		[ 'SCOPE_LABEL_MOUNTED_NFS', ic.FSType.Mounted, ic.FSName.Nfs   ],

		[ 'SCOPE_LABEL_MOUNTED', ic.FSType.AllRemotelyMounted, ic.FSName.Any ],

		[ 'SCOPE_LABEL_SHARED_SMB',  ic.FSType.Shared,  ic.FSName.Samba ],
		[ 'SCOPE_LABEL_SHARED_NFS',  ic.FSType.Shared,  ic.FSName.Nfs   ],

		[ 'SCOPE_LABEL_SHARED',  ic.FSType.AllShared, ic.FSName.Any ]
	];

	this.convertData = convertData;

	this.findData = function( $scopeType )
	{
		for(var i in convertData)
			if(i == $scopeType.val())
				return convertData[i];

		throw "Couldn't find data";
	}

	function pathMustBeEmpty(fstype)
	{
		return((fstype == ic.FSType.AllRemotelyMounted)
			|| (fstype == ic.FSType.AllShared));
	}

	function pathCanBeEmpty(fstype)
	{
		return((fstype == ic.FSType.AllRemotelyMounted)
			|| (fstype == ic.FSType.AllShared)
			|| (fstype == ic.FSType.Shared)
			|| (fstype == ic.FSType.Mounted));
	}

	function pathMustStartWithRoot( $scopeType, $path )
	{
		var data = this_.findData( $scopeType );
		var fstype = data[1];
		var fsname = data[2];

		if(fstype == ic.FSType.Local)
			return true;

		if($path.val().length == 0) // not local and empty path
			return false;

		// Only samba share might not start with /
		if((fsname == ic.FSName.Samba) && (fstype == ic.FSType.Shared))
			return false;

		return true;
	}

	this.findDataByObject = function( areaPathObject, clbk )
	{
		for(var i in convertData){
			var data   = convertData[i];
			var fstype = data[1];
			var fsname = data[2];

			if(fstype == areaPathObject.FSType && fsname == areaPathObject.FSName){
				clbk(i, data, fstype, fsname);
				break;
			}
		}
	}

	function syncView( $scopeType, $scopePath, areaPathObject )
	{
		this_.findDataByObject( areaPathObject, function(i){
			$scopeType.val(i);
			$scopePath.val(areaPathObject.Path);
		})
	}

	function syncObject( $scopeType, $scopePath, areaPathObject )
	{
		var data = this_.findData( $scopeType );
		areaPathObject.FSType = data[1];
		areaPathObject.FSName = data[2];

		if(pathMustBeEmpty(data[1]))
			$scopePath.val('');

		areaPathObject.Path = $scopePath.val();
	}

	this.createOptionsHtml = function( $scopeType )
	{
		$scopeType.empty()
		convertData.each(function(item, i){
			$scopeType.append('<option value="' + i + '">' + _res(item[0]) + '</option>');
		})
	}

	this.validateScanScopePath = function( $type, $path )
	{
		if(pathMustStartWithRoot($type, $path) && ($path.val().indexOf('/') != 0))
			$path.val('/'+$path.val())

		return true;
	}

	function setupPathActivity( $path, type )
	{
		if(pathMustBeEmpty(type))
			$path.disable().val('');
		else
			$path.enable();
	}

	function checkENTERKey( $path, $ok, e )
	{
		if($path.val() && (typeof(e.which) !== 'undefined') && (e.which === 13))
			$ok.click();
	}

	this.bind = function( $type, $path, $ok,
		getAreaPath,
		enableOk,
		disableOk,
		onSyncCallback )
	{
		function checkState(e)
		{
			var t = this_.findData($type)[1];

			setupPathActivity($path, t);

			if(pathCanBeEmpty(t) || $path.val()) {
				enableOk();
				checkENTERKey( $path, $ok, e );
			}
			else
				disableOk();
		}
		this_.createOptionsHtml($type);
		syncView($type, $path, getAreaPath());

		$type.change(checkState).change();
// 		$path.keyup(checkState).focus();
		$path.fastChange(checkState).focus();

		$ok.click(function(){
			if(!this_.validateScanScopePath($type, $path))
				return;

			syncObject($type, $path, getAreaPath());
			onSyncCallback();
		});
	}
};

var scopePage_synchronizer = new scopePage_ViewDataSynchronizer();

function scopePage_onLoad()
{
	$('#tableArea').empty();

	var isIncludeScope = (window.location.hash.indexOf('__scope__') > -1);
	var isExcludeScope = (window.location.hash.indexOf('__excludescope__') > -1);

	if(!isIncludeScope && !isExcludeScope)
		return;

	// init global variable
	if(!this.scopePage_model)
		this.scopePage_model = new ScanScopeModel();

	scopePage_model.bindDomElement = 'tableArea';

	taskDesc = describeTaskByLocation();

	var isOAS = (taskDesc.clazz == ic.TaskClass.OAS);
	var isODS = (taskDesc.clazz == ic.TaskClass.ODS);

	scopePage_model.enableUsers = isOAS;

	if(isIncludeScope)
	{
		scopePage_model.enableScanSettings = true;

		if(isOAS)
			scopePage_model.scopeItemCreator = function() { return $.extend(true, {}, ic.create_OASSettings_ScanArea(), {
				ScanSettings: {
					UseAdvancedActions: true,
					AdvancedActions: [{FirstAction:5, SecondAction:5, Verdict:1, __VersionInfo:"1 0"}]
				}
			})};
		else if(isODS)
			scopePage_model.scopeItemCreator = function() { return $.extend(true, {}, ic.create_ODSSettings_ScanArea(), {
				ScanSettings: {
					UseAdvancedActions: true,
					AdvancedActions: [{FirstAction:5, SecondAction:5, Verdict:1, __VersionInfo:"1 0"}]
				}
			})};

		function __setIncludeScopes()
		{
			scopePage_model.scopes = editTask.SettingsTask.ScanScope;
			taskSettings_addRevertFunc(__setIncludeScopes);
		}
		__setIncludeScopes();
	}
	else if(isExcludeScope)
	{
		function __setExcludeScopes()
		{
			scopePage_model.scopes = editTask.SettingsTask.ExcludedFromScanScope;
			taskSettings_addRevertFunc(__setExcludeScopes);
		}
		__setExcludeScopes();

		if(isOAS)
			scopePage_model.scopeItemCreator = ic.create_OASSettings_ExcludedFromScanArea;
		else if(isODS)
			scopePage_model.scopeItemCreator = ic.create_ODSSettings_ExcludedFromScanArea;
	}
	else
		return;

	scopePage_model.run();
}
