
/*  
 * This source file is a part of a Kaspersky Antivirus Monitor.
 * Copyright (C) Kaspersky Lab, 1997-2010
 * See License.txt for details
 *
 */

#ifndef KAVMON_QUEUE_H
#define KAVMON_QUEUE_H

typedef struct
{
	int						refcnt;
	lock_declare(lock);
	u_short					queue_id;
	int						flags;
	short					blocked;
	int		                answer;
	wait_queue_head_t		waitq;
	struct list_head		list;
} mqueue_entry_t;

#define O_MQUEUE_READY			0x02
#define O_MQUEUE_TESTING		0x04

int  Monitor_queue_init(void);
int  Monitor_queue_get_id(void);
void Monitor_queue_set_answer(u_short queue_id, int answer);

void Monitor_queue_wakeup(u_short queue_id);
void Monitor_queue_cleanup(void);
#ifndef __linux__
int Monitor_queue_add(u_short queue_id, int block, PackedBuffer * request, enum FileAccessType  * answer);
#else /*__linux__*/
typedef struct
{
    // common fields for all (CHECK_SIGN and CHECK_FILE) requests
    struct list_head            list;
    atomic_t                    refcnt;
    u_int32_t                   euid;            // effective user ID
    u_int32_t                   egid;            // effective group ID
    const char *                filename;
    u_int16_t                   queue_id;       //used only in MONITOR, but probably will be used in SAMBA in the future
    u_int8_t                    request;
    // additional fields only for CHECK_FILE requests
    u_int8_t                    file_op_type;      //enum file_oper_type
    u_int32_t                   pid;            //process ID
    int32_t                     flags;          //flags like O_RDONLY etc
    struct vfsmount *           rootmnt;        //the process
    struct dentry *             root;           //  root
    dev_t                       dev;
    ino_t                       ino;
    time_t                      mtime;
    int64_t                     size;
    u_int32_t                   owner_uid;
    u_int32_t                   file_mode;
    int64_t                     f_type;         //file system type
} check_req_data_t;

int Monitor_queue_add(u_short queue_id, int block, check_req_data_t * reqd, enum FileAccessType  * answer);

static inline void check_req_init(check_req_data_t *reqd, u_int8_t request)
{
    atomic_set(&reqd->refcnt, 1);
    reqd->request = request;
}

static inline void fill_check_req_common_fields(check_req_data_t *reqd, u_int32_t euid,
        u_int32_t egid, u_int8_t request, const char *filename)
{
    check_req_init(reqd, request);
    reqd->euid = euid;
    reqd->egid = egid;
    reqd->filename = filename;
}

static inline void fill_check_sign_req(check_req_data_t *reqd, u_int32_t euid,
        u_int32_t egid, u_int8_t request, const char *filename)
{
    fill_check_req_common_fields(reqd, euid, egid, request, filename);
    reqd->rootmnt = NULL;
    reqd->root = NULL;
}

extern int check_req_queue_init(void);
extern void check_req_queue_add(check_req_data_t *reqd);
extern check_req_data_t *check_req_queue_get(void);
extern void check_req_queue_put(check_req_data_t *reqd);
extern void check_req_queue_release(check_req_data_t *reqd);
extern int check_req_queue_empty(void);
#endif /*__linux__*/

#endif

